unit uOrder;

interface

uses
  System.Classes, System.StrUtils, System.DateUtils, Generics.Collections,
  Bcl.Validation.Attributes, Bcl.Validation.Classes, Aurelius.Types.Blob,
  Aurelius.Types.Nullable, Aurelius.Types.Proxy, Aurelius.Mapping.Attributes,

  // Entidades
  uStore, uCustomer, uProduct, uUser, uDeliver;

type
  // Estado de product
  [Enumeration(TEnumMappingType.emChar, 'P,G,M,L,F')]
  TStatusOrder = (Pendiente, Pagado, MuestraRecibida, EnLaboratorio, Finalizado);

  TItem = class;


  [Entity, Automapping]
  [Table('OrderMaster')]
  [Id('FIdOrderMaster', TIdGenerator.IdentityOrSequence)]
  TOrderMaster = class
  private
    [Column('IdOrderMaster', [TColumnProp.Required])]
    FIdOrderMaster: integer;

    [Column('DateTime')]
    FDateTime: TDateTime;

    [Column('StatusOrder', [], 1)]
    FStatusOrder: TStatusOrder;

    [ForeignKey('FK_OrderMaster2Store')]
    [JoinColumn('IdStore', [TColumnProp.Lazy, TColumnProp.Required])]
    FStore: TStore;

    [ForeignKey('FK_OrderMaster2Customer')]
    [JoinColumn('IdCustomer', [TColumnProp.Lazy, TColumnProp.Required])]
    FCustomer: TCustomer;

    [ForeignJoinColumn('IdOrder', [TColumnProp.Required])]
    FItems: Proxy<TList<TItem>>;

    [ForeignKey('FK_OrderMaster2User')]
    [JoinColumn('IdUser', [TColumnProp.Lazy])]
    FUser: TUser;

    [ForeignKey('FK_OrderMaster2DeliverMethod')]
    [JoinColumn('IdDeliverMethod', [TColumnProp.Lazy, TColumnProp.Required])]
    FDeliverMethod: TDeliverMethod;

    [Column('Memo', [], 300)]
    FMemo: Nullable<string>;


    // Pickup por tecnicos de laboratorio
    [Column('PickupLabTechDateFrom')]
    FPickupLabTechDateFrom: Nullable<TDate>;

    [Column('PickupLabTechDateTo')]
    FPickupLabTechDateTo: Nullable<TDate>;


    [Column('PickupLabTechTimeFrom')]
    FPickupLabTechTimeFrom: Nullable<TTime>;

    [Column('PickupLabTechTimeTo')]
    FPickupLabTechTimeTo: Nullable<TTime>;


    [Column('PickupLabTechAddress', [], 200)]
    FPickupLabTechAddress: Nullable<string>;


    // Pickup por delivery
    [Column('PickupDeliveryDate')]
    FPickupDeliveryDate: Nullable<TDate>;

    [Column('PickupDeliveryTimeFrom')]
    FPickupDeliveryTimeFrom: Nullable<TTime>;

    [Column('PickupDeliveryTimeTo')]
    FPickupDeliveryTimeTo: Nullable<TTime>;

    [Column('PickupDeliveryAddress', [], 200)]
    FPickupDeliveryAddress: Nullable<string>;

    [Column('PickupDeliveryInstructions', [], 200)]
    FPickupDeliveryInstructions: Nullable<string>;

    [ForeignKey('FK_Order2TypeProduct')]
    [JoinColumn('IdTypeProduct', [TColumnProp.Lazy{, TColumnProp.Required}])]
    FTypeProduct: TTypeProduct;

    function GetItems: TList<TItem>;
    function GetTotalOrder: Currency;

  public
    constructor Create; overload;
    destructor Destroy; override;

    property IdOrderMaster: integer read FIdOrderMaster write FIdOrderMaster;
    property DateTime: TDateTime read FDateTime write FDateTime;
    property StatusOrder: TStatusOrder read FStatusOrder write FStatusOrder;

    property Store: TStore read FStore write FStore;
    property Customer: TCustomer read FCustomer write FCustomer;
    property User: TUser read FUser write FUser;
    property DeliverMethod: TDeliverMethod read FDeliverMethod write FDeliverMethod;

    property Memo: Nullable<string> read FMemo write FMemo;

    // Recoleccion de tecnico de laboratorio
    property PickupLabTechDateFrom: Nullable<TDate> read FPickupLabTechDateFrom write FPickupLabTechDateFrom;
    property PickupLabTechDateTo: Nullable<TDate> read FPickupLabTechDateTo write FPickupLabTechDateTo;
    property PickupLabTechTimeFrom: Nullable<TTime> read FPickupLabTechTimeFrom write FPickupLabTechTimeFrom;
    property PickupLabTechTimeTo: Nullable<TTime> read FPickupLabTechTimeTo write FPickupLabTechTimeTo;

    property PickupLabTechAddress: Nullable<string> read FPickupLabTechAddress write FPickupLabTechAddress;

    // Recoleccion por delivery
    property PickupDeliveryDate: Nullable<TDate> read FPickupDeliveryDate write FPickupDeliveryDate;
    property PickupDeliveryTimeFrom: Nullable<TTime> read FPickupDeliveryTimeFrom write FPickupDeliveryTimeFrom;
    property PickupDeliveryTimeTo: Nullable<TTime> read FPickupDeliveryTimeTo write FPickupDeliveryTimeTo;

    property PickupDeliveryAddress: Nullable<string> read FPickupDeliveryAddress write FPickupDeliveryAddress;
    property PickupDeliveryInstructions: Nullable<string> read FPickupDeliveryInstructions write FPickupDeliveryInstructions;

    property TypeProduct: TTypeProduct read FTypeProduct write FTypeProduct;

    // Items
    property Items: TList<TItem> read GetItems;
    property TotalOrder: Currency read GetTotalOrder;
  end;


  [Entity, Automapping]
  [Table('Item')]
  [Id('FIdItem', TIdGenerator.IdentityOrSequence)]
  TItem = class
  private
    [Column('IdItem', [TColumnProp.Required])]
    FIdItem: integer;

    [ForeignKey('FK_Item2Product')]
    [JoinColumn('IdProduct', [TColumnProp.Lazy, TColumnProp.Required])]
    FProduct: TProduct;

    [Column('Quantity', [])]
    FQuantity: Double;

    [Column('UnitPrice', [])]
    FUnitPrice: Currency;
    function GetTotal: Currency;
  public
    property IdItem: integer read FIdItem write FIdItem;
    property Product: TProduct read FProduct write FProduct;
    property Quantity: Double read FQuantity write FQuantity;
    property UnitPrice: Currency read FUnitPrice write FUnitPrice;
    property Total: Currency read GetTotal;
  end;



implementation



{ TOrder }

constructor TOrderMaster.Create;
begin
  FItems.SetInitialValue(TList<TItem>.Create);
end;

destructor TOrderMaster.Destroy;
begin
  FItems.DestroyValue;
  inherited;
end;


function TOrderMaster.GetItems: TList<TItem>;
begin
  Result := FItems.Value;
end;


function TOrderMaster.GetTotalOrder: Currency;
var
  i: integer;
begin
  Result := 0;

  // Recorro los items
  for i := 0 to FItems.Value.Count - 1 do
  begin
    Result := FItems.Value[i].Quantity * FItems.Value[i].UnitPrice;
  end;
end;



{ TItem }

function TItem.GetTotal: Currency;
begin
  Result := Quantity * UnitPrice;
end;

initialization
  RegisterEntity(TOrderMaster);
  RegisterEntity(TItem);
end.
